require 'logger'
require 'optparse'
require 'irb'

module Mooset
  class Application
    attr_accessor :config_filename, :interactive, :logfile

    def define(endpoint_name, factory, opts = {})
      opts[:endpoint_name] = endpoint_name

      endpoints[endpoint_name] = Mooset::Endpoints::Endpoint.create(factory, opts)
    end

    def endpoints
      @endpoints ||= {}
    end

    def instances
      @instances ||= {}
    end

    def synchronize_group(from:, to:, **opts)
      operations << Mooset::SynchronizeGroup.new(endpoints[from], endpoints[to], opts)
    end

    def operations
      @operations ||= []
    end

    def run!
      operations.each do |operation|
        operation.call
      end
    end

    def start_irb
      $app = self

      IRB.start
    end

    def read_config
      instance_eval File.read(config_filename), config_filename
    end

    def configure(&block)
      yield self
    end

    def self.run(argv)
      s = self.new

      OptionParser.new do |opts|
        opts.banner = "Usage: #{$0} [options]"
        opts.on("-f", "--logfile FILE", "Logfile", s.method(:logfile=))
        opts.on("-v", "--verbose", "Verbose") do |x|
          s.logger.level = ::Logger::DEBUG
        end

        opts.on("-c", "--config FILE", "Config file", s.method(:config_filename=))
        opts.on("-i", "--interactive", "Run in interactive mode", s.method(:interactive=))
        opts.parse!(argv)
      end

      LSpace[:logger] = s.logger

      if s.config_filename
        s.read_config
        s.run! unless s.interactive
      end

      s.start_irb if s.interactive
    end

    def logger
      @logger ||= ::Logger.new(logfile || STDOUT).tap do |l|
        l.level = ::Logger::INFO
      end
    end

  end
end
