require 'virtus'

module Mooset
  module Endpoints
    class Endpoint
      include Virtus.model

      attribute :logger
      attribute :endpoint_name

      def self.inherited(klass)
        super

        name = underscore(klass.name.split('::').last)
        descendants[name.to_sym] = klass
      end

      def self.descendants
        @descendants ||= {}
      end

      def self.underscore(camel_cased_word)
        camel_cased_word.to_s.gsub(/::/, '/').
          gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
          gsub(/([a-z\d])([A-Z])/,'\1_\2').
          tr("-", "_").
          downcase
      end

      def self.create(name, opts = {})
        descendants[name].new(opts)
      end

      def self.resources
        @resources ||= []
      end

      def self.resource(name, &block)
        resources << name

        define_method(name) do
          r = instance_variable_get("@#{name}")
          if r.nil?
            r = instance_eval(&block)
            instance_variable_set("@#{name}", r)
          end
          r
        end

      end
    end
  end
end
