module Mooset
  module Endpoints
    class Gitlab
      class Group < Models::Group
        GUEST     = 10
        REPORTER  = 20
        DEVELOPER = 30
        MASTER    = 40
        OWNER     = 50

        attribute :gitlab_id
        attribute :path
        attribute :owner_id
        attribute :provider, String, default: "gitlab"

        finders [:gitlab_id, :name]

        def self.build(endpoint, group)
          new(
            endpoint: endpoint,
            id: group.id,
            gitlab_id: group.id,
            name: group.name,
            path: group.path,
            owner_id: group.owner_id
          )
        end

        def members
          @members ||= endpoint.users.group_members(id).to_a
        end

        def <<(user)
          logger.info "#{self} << #{user}"

          endpoint.groups.add_group_member(id, user.id, DEVELOPER)
        end

        def add_member(user, role)
          logger.info "#{self} << #{user} #{role}"

          endpoint.groups.add_group_member(id, user.id, role)
        end
      end
    end
  end
end
