module Mooset
  module Endpoints
    class Gitlab
      class GroupSearch < Resource
        include Paginate

        attr_reader :endpoint

        def initialize(endpoint)
          @endpoint = endpoint
        end

        def all
          return to_enum(:all) unless block_given?

          paginate ->(page){ connection.groups(page: page) }, ->(object){ yield Group.build(endpoint, object) }
        end

        finder :all do |_, opts = {}|
          all
        end

        finder :id do |id, opts = {}|
          Group.build(endpoint, connection.group(id))
        end

        finder :gitlab_id do |gitlab_id, opts = {}|
          find(id: gitlab_id)
        end

        finder :name do |name, opts = {}|
          all.find { |x| x.name == name }
        end

        import :ldap do |group, opts = {}|
          create(group.name, group.name.gsub(/( )/, '_').downcase)
        end

        def members(id)
          return to_enum(:members, id) unless block_given?

          paginate ->(page) { connection.group_members(id, page: page).map { |object| yield User.build(endpoint, object) } }
        end

        def create(name, path)
          Group.build(endpoint, connection.create_group(name, path))
        end

        def add_group_member(team_id, user_id, access_level = 50)
          connection.add_group_member(team_id, user_id, access_level)
        end

        private

        def connection
          endpoint.connection
        end
      end
    end
  end
end

