module Mooset
  module Endpoints
    class Gitlab
      class User < Models::User
        attribute :gitlab_id
        attribute :state
        attribute :access_level
        attribute :provider, String, default: "gitlab"
        attribute :ldap_id, String

        finders [:gitlab_id, :ldap_id, :email, :username]

        def self.build(endpoint, user)
          self.new(
            endpoint: endpoint,
            id: user.id,
            gitlab_id: user.id,
            username: user.username,
            full_name: user.name,
            state: user.state,
            email: user.email,
            access_level: user.access_level,
            ldap_id: Optional.new(user).identities.within do |identities|
              identity_from(Many.new(identities)).values.first
            end.value,
          )
        end

        def self.identity_from(identities)
          identities.within do |o|
            o["extern_uid"] if ["ldap", "ldapmain"].include?(o["provider"])
          end
        end
      end
    end
  end
end

