module Mooset
  module Endpoints
    class Gitlab
      class UserSearch < Resource
        include Paginate

        attr_reader :endpoint

        def initialize(endpoint)
          @endpoint = endpoint
        end

        def all
          return to_enum(:all) unless block_given?

          paginate ->(page){ connection.users(page: page) }, ->(object){ yield User.build(endpoint, object) }
        end

        def alll
          @alll ||= all.to_a
        end

        finder :id do |id, opts = {}|
          User.build(endpoint, connection.user(id))
        end

        finder :gitlab_id do |gitlab_id, opts = {}|
          find(id: gitlab_id)
        end

        finder :username do |username, opts = {}|
          alll.find { |x| x.username == username }
        end

        finder :email do |email, opts = {}|
          alll.find { |x| x.email == email }
        end

        finder :dn do |dn, opts = {}|
          alll.find { |x| x.ldap_id == dn }
        end

        import :ldap do |user|
          create(user.email, SecureRandom.hex(8), {
            name: user.full_name,
            username: user.username,
            extern_uid: user.dn,
            provider: endpoint.provider,
            confirm: false
          })
        end

        def create(email, password, opts = {})
          User.build(endpoint, connection.create_user(email, password, opts))
        end

        def group_members(id)
          return to_enum(:group_members, id) unless block_given?

          paginate ->(page) { connection.group_members(id, page: page) }, ->(object) { yield User.build(endpoint, object) }
        end

        private

        def connection
          endpoint.connection
        end
      end
    end
  end
end

