require 'net/ldap'

module Mooset
  module Endpoints
    class Ldap < Endpoint
      USER_FILTER = "(objectClass=person)"
      GROUP_FILTER = "(objectClass=group)"

      attribute :host, String
      attribute :port, Integer
      attribute :bind_dn, String
      attribute :bind_password, String
      attribute :treebase, String
      attribute :encryption, Symbol
      attribute :connection, Object, default: Proc.new { |ldap, attribute|
        connection = Net::LDAP.new(encryption: ldap.encryption)
        connection.host = ldap.host
        connection.port = ldap.port
        connection.auth ldap.bind_dn, ldap.bind_password
        connection
      }

      resource :users do
        LdapSearch.new(self, "(objectClass=person)")
      end

      resource :groups do
        LdapSearch.new(self, "(|(objectClass=group)(objectClass=organizationalUnit))")
      end
    end
  end
end

