module Mooset
  module Endpoints
    class Ldap
      class LdapSearch < Resource
        GROUP_FILTER = "(objectClass=group)"

        attr_reader :endpoint

        def initialize(endpoint, default_search)
          @endpoint = endpoint
          @default_search = default_search
        end

        finder :dn do |dn, base: nil|
          s = base.nil? ? @default_search : dn
          b = base.nil? ? dn : base

          query(s, base: b)
        end

        finder :name do |name, opts = {}|
          query("(&(objectclass=group)(cn=#{name}))")
        end

        finder :memberof do |dn, base: endpoint.treebase|
          query("(&(objectclass=person)(memberof=#{dn})(!(useraccountcontrol:1.2.840.113556.1.4.803:=2)))", base: base)
        end

        def query(filter = @default_search, base: endpoint.treebase)
          return to_enum(:query, filter, base: base) unless block_given?

          connection.search(base: base, filter: filter) do |object|
            if object[:objectclass] && object[:objectclass].include?("group")
              r = Group.build(endpoint, object)
            elsif object[:objectclass] && object[:objectclass].include?("user")
              r = User.build(endpoint, object)
            elsif object[:objectclass] && object[:objectclass].include?("organizationalUnit")
              r = Ou.build(endpoint, object)
            else
              raise "Unknown objectclass #{object[:objectclass]}"
            end

            yield r
          end
        end

        private

        def connection
          @endpoint.connection
        end
      end
    end
  end
end
