module Mooset
  module Endpoints
    class Ldap
      module AccountControl
        SCRIPT                         = 0x0001
        ACCOUNTDISABLE                 = 0x0002
        HOMEDIR_REQUIRED               = 0x0008
        LOCKOUT                        = 0x0010
        PASSWD_NOTREQD                 = 0x0020
        PASSWD_CANT_CHANGE             = 0x0040
        ENCRYPTED_TEXT_PWD_ALLOWED     = 0x0080
        TEMP_DUPLICATE_ACCOUNT         = 0x0100
        NORMAL_ACCOUNT                 = 0x0200
        INTERDOMAIN_TRUST_ACCOUNT      = 0x0800
        WORKSTATION_TRUST_ACCOUNT      = 0x1000
        SERVER_TRUST_ACCOUNT           = 0x2000
        DONT_EXPIRE_PASSWORD           = 0x10000
        MNS_LOGON_ACCOUNT              = 0x20000
        SMARTCARD_REQUIRED             = 0x40000
        TRUSTED_FOR_DELEGATION         = 0x80000
        NOT_DELEGATED                  = 0x100000
        USE_DES_KEY_ONLY               = 0x200000
        DONT_REQ_PREAUTH               = 0x400000
        PASSWORD_EXPIRED               = 0x800000
        TRUSTED_TO_AUTH_FOR_DELEGATION = 0x1000000
        PARTIAL_SECRETS_ACCOUNT        = 0x04000000

        class << self
          def decode(value)
            value.within do |int|
              result = OpenStruct.new

              AccountControl.constants(false).each do |name|
                result[name.to_s.downcase] = (int & AccountControl.const_get(name)) > 0
              end

              result
            end.value
          end
        end
      end

      class User < Models::User
        finders [:dn, :username, :email]

        attribute :provider, String, default: "ldap"
        attribute :dn, String

        def self.build(endpoint, params)
          user = Optional.new(params)

          self.new(
            endpoint: endpoint,
            id: user[:dn].first.value,
            dn: user[:dn].first.value,
            email: user[:mail].first.value,
            full_name: user[:cn].first.value,
            username: user[:sAMAccountName].first.value,
          )
        end
      end
    end
  end
end
