module Mooset
  module Findable
    module ClassMethods
      def import(name, &block)
        define_method(import_method_name(name), &block)
      end

      def finder(name, &block)
        define_method(finder_method_name(name), &block)
      end

      def finder_method_name(name)
        "find_by_#{name}"
      end

      def import_method_name(name)
        "import_from_#{name}"
      end
    end

    module InstanceMethods
      def import(object)
        logger.debug "import #{object}"

        method = self.class.import_method_name(object.provider)

        public_send(method, object)
      end

      def find_from_object(object, *args)
        finder = object.finders.find { |x| respond_to? self.class.finder_method_name(x) }

        if finder && object[finder]
          public_send(self.class.finder_method_name(finder), object[finder], *args)
        end
      end

      def find_from_hash(hash)
        key, value = hash.find { |(k, _)| respond_to? self.class.finder_method_name(k) }

        args = hash.tap do |h|
          h.delete(key)
        end

        if key && value
          public_send(self.class.finder_method_name(key), value, args)
        end
      end

      def find(opts, *args)
        logger.debug "find #{opts}"

        return find_from_hash(opts, *args) if opts.is_a? Hash

        find_from_object(opts, args)
      end
    end

    def self.included(receiver)
      receiver.extend         ClassMethods
      receiver.send :include, InstanceMethods
    end
  end
end
