module Mooset
  module Monad
    def within(&block)
      and_then do |value|
        self.class.from_value(block.call(value))
      end
    end

    def method_missing(*args, &block)
      within do |value|
        begin
          value.public_send(*args, &block)
        rescue
          require 'pry'; binding.pry
        end
      end
    end

    private

    def ensure_monadic_result(&block)
      acceptable_result_type = self.class

      ->(*a, &b) do
        block.call(*a, &b).tap do |result|
          unless result.is_a?(acceptable_result_type)
            raise TypeError, "block must return #{acceptable_result_type.name}"
          end
        end
      end
    end
  end

  class Optional
    include Monad

    attr_reader :value
    def initialize(value)
      @value = value
    end

    def and_then(&block)
      block = ensure_monadic_result(&block)

      if value.nil?
        self
      else
        block.call(value)
      end
    end

    def self.from_value(value)
      Optional.new(value)
    end
  end

  class Many
    include Monad

    attr_reader :values

    def initialize(values)
      @values = values
    end

    def and_then(&block)
      block = ensure_monadic_result(&block)

      Many.new(values.map(&block).flat_map(&:values))
    end

    def self.from_value(value)
      Many.new([value])
    end
  end
end
